# -*- coding: utf-8 -*-
import syslog

from pylons.i18n import _
from ckan.logic.auth import get_package_object, get_resource_object
import ckan.logic.auth
import ckan.plugins as p
from ckan.lib.base import h
from ckanext.dgu.lib import helpers as dgu_helpers
import ckan.logic as logic

def opsi_group_update(context, data_dict):
    model = context['model']
    user = context.get('user')
    user_obj = model.User.get( user )
    
    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}

    return ckan.logic.auth.update.group_update(context, data_dict)
    
def opsi_organization_update(context, data_dict):
    model = context['model']
    user = context.get('user')
    user_obj = model.User.get( user )

    # Allow sysadmins to create anything
    if user_obj.sysadmin:
        return {'success': True}
    
    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}
    
    # Sysadmins only
    return { 'success': False, 'msg': 'Le sistemski administratorji in glavni uredniki lahko urejajo organizacije' }

def opsi_package_change_state(context, data_dict):
    model = context['model']
    user = context.get('user')
    user_obj = model.User.get( user )

    # Allow sysadmins to create anything
    if user_obj.sysadmin:
        return {'success': True}

    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}

    # Sysadmins or executive editors only
    return { 'success': False, 'msg': 'Le sistemski administratorji in glavni uredniki lahko neposredno spreminjajo stanje zbirke' }

def opsi_package_create(context, data_dict):
    model = context['model']
    user = context.get('user')
    user_obj = model.User.get( user )

    # Allow sysadmins to create anything
    if user_obj.sysadmin:
        return {'success': True}
    
    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}
    
    # "Vsebinski urednik" belonging to an organization
    if dgu_helpers.is_content_editor():
        if data_dict and data_dict.get('state') == 'active':
            return {'success': False,
                    'msg': _('Uporabnik %s nima pravic za objavo zbirke %s') %
                    (str(user), package.id)}

        q = model.Session.query(model.Member) \
            .filter(model.Member.table_name == 'user') \
            .filter(model.Member.state == 'active') \
            .filter(model.Member.table_id == user_obj.id)
        group_ids = []
        for row in q.all():
            group_ids.append(row.group_id)
            
        q = model.Session.query(model.Group) \
            .filter(model.Group.is_organization == True) \
            .filter(model.Group.state == 'active') \
            .filter(model.Group.id.in_(group_ids))
        
        if q.count():
            return {'success': True}
        else:
            # Message must not be unicode! CKAN logger uses ascii and produces an error if non-ascii characters are used in msg.
            return {'success': False, 'msg': _('Ne pripadate nobeni organizaciji. Za dodelitev pravic vas prosimo, da izpolnete <a href="%s">obrazec</a>.') % (h.url_for(controller='ckanext.dgu.controllers.publisher:PublisherController', action='apply'))}

    # Leave the core CKAN auth to work out the hierarchy stuff
    return ckan.logic.auth.create.package_create(context, data_dict)

@p.toolkit.auth_allow_anonymous_access
def opsi_package_show(context, data_dict):
    # Odgovorni urednik can view all datasets, even deleted ones
    if dgu_helpers.is_executive_editor():
        return {'success': True}
    
    return ckan.logic.auth.get.package_show(context, data_dict)

def opsi_resource_create(context, data_dict):
    model = context['model']
    user = context.get('user')
    
    package_id = data_dict.get('package_id')
    if not package_id and data_dict.get('id'):
        # This can happen when auth is deferred, eg from `resource_view_create`
        resource = get_resource_object(context, data_dict)
        package_id = resource.package_id

    if not package_id:
        raise ckan.logic.NotFound(
            _('No dataset id provided, cannot check auth.')
        )
    
    # check authentication against package
    data_dict['package'] = model.Package.get(package_id)
    if not data_dict['package']:
        raise ckan.logic.NotFound(
            _('No package found for this resource, cannot check auth.')
        )
    
    if not data_dict.get('id'):
        data_dict['id'] = package_id
    authorized = opsi_package_update(context, data_dict).get('success')

    if not authorized:
        return {'success': False,
                'msg': _('User %s not authorized to create resources on dataset %s') %
                        (str(user), package_id)}
    else:
        return {'success': True}

def opsi_resource_update(context, data_dict):
    model = context['model']
    user = context.get('user')

    resource = get_resource_object(context, data_dict)

    query = model.Session.query(model.Package)\
        .join(model.ResourceGroup)\
        .join(model.Resource)\
        .filter(model.ResourceGroup.id == resource.resource_group_id)
    pkg = query.first()
    if not pkg:
        raise logic.NotFound(
            _('No package found for this resource, cannot check auth.')
        )

    # check authentication against package
    pkg_dict = { 'id': pkg.id }
    authorized = opsi_package_update(context, pkg_dict).get('success')

    if not authorized:
        return {'success': False,
                'msg': _('User %s not authorized to update resources on dataset %s') %
                        (str(user), pkg.id)}
    else:
        return {'success': True}
    
def opsi_resource_delete(context, data_dict):
    # check authentication against package
    model = context['model']
    resource = get_resource_object(context, data_dict)
    query = model.Session.query(model.Package) \
        .join(model.ResourceGroup) \
        .join(model.Resource) \
        .filter(model.ResourceGroup.id == resource.resource_group_id)
    pkg = query.first()
    if not pkg:
        raise logic.NotFound(_('No package found for this resource, cannot check auth.'))

    pkg_dict = {'id': pkg.id}
    # package update needs package id, not resource id
    authorized = opsi_package_update(context, pkg_dict)

    if not authorized:
        return {'success': False,
                'msg': _('User %s not authorized to delete resources on dataset %s') %
                        (str(user), pkg.id)}
    else:
        return {'success': True}

def opsi_package_update(context, data_dict):
    model = context['model']
    user = context.get('user')
    user_obj = model.User.get( user )
    package = get_package_object(context, data_dict)

    # Allow sysadmins to edit anything
    # This includes UKLP harvested datasets.
    #   Note: the harvest user *is* a sysadmin
    #   Note: if changing this, check the code and comments in
    #         ckanext/forms/dataset_form.py:DatasetForm.form_to_db_schema_options()
    if user_obj.sysadmin:
        return {'success': True}
    
    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}
    
    # "Vsebinski urednik", member of organization that is the owner of this package
    if dgu_helpers.is_content_editor():
        # reject saving active state unless it was already active and it is being saved as unpublished
        # Note: resource_create initial auth checking skips this check due to state not being in data_dict of resource
        if data_dict and data_dict.get('state') == 'active' and (package.state != 'active' or data_dict.get('unpublished', 'False') != 'True'):
            return {'success': False,
                    'msg': _('Uporabnik %s nima pravic za objavo zbirke %s') %
                    (str(user), package.id)}

        if package.owner_org:
            q = model.Session.query(model.Member) \
                .filter(model.Member.table_name == 'user') \
                .filter(model.Member.state == 'active') \
                .filter(model.Member.table_id == user_obj.id) \
                .filter(model.Member.group_id == package.owner_org)
            if q.all():
                return {'success': True}
            else:
                return {'success': False,
                        'msg': _('User %s not authorized to edit package %s') %
                        (str(user), package.id)}
        else:
            return opsi_package_create(context, data_dict)

    # UKLP datasets and other harvested datasets cannot be edited by the
    # average admin/editor because changes will be overwritten on next harvest.
    if dgu_helpers.was_dataset_harvested(package.extras):
        return {'success': False,
                'msg': _('User %s not authorized to edit harvested datasets') % str(user)}

    # Leave the core CKAN auth to work out the hierarchy stuff
    return ckan.logic.auth.update.package_update(context, data_dict)

def opsi_dataset_delete(context, data_dict):
    """
    Determines whether a dataset's state can be set to "deleted".

    Currently only sysadmin users can do this, apart from UKLP.
    """
    model = context['model']
    user = context.get('user')
    if not user:
        return {'success': False}
    user_obj = model.User.get(user)
    package = get_package_object(context, data_dict)

    # Sysadmin can delete any package, including UKLP
    if user_obj.sysadmin:
        return {'success': True}

    # Don't allow admin/editor to delete
    # * apart from UKLP datasets which CAN be withdrawn by the appropriate
    # admin/editor because they are all live services, so can't be cached to
    # preserve them without the service provider's help
    #if package.extras.get('UKLP', '') != 'True':
    #    return {'success': False}
    
    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}
    
    # "Vsebinski urednik", member of organization that is the owner of this package
    if dgu_helpers.is_content_editor():
        authorized = False
        if package.owner_org:
            q = model.Session.query(model.Member) \
                .filter(model.Member.table_name == 'user') \
                .filter(model.Member.state == 'active') \
                .filter(model.Member.table_id == user_obj.id) \
                .filter(model.Member.group_id == package.owner_org)
            authorized = bool(q.all())
        else:
            authorized = opsi_package_create(context, data_dict).get('success')
        if authorized:
            return {'success': True}
        else:
            return {'success': False,
                    'msg': _('User %s not authorized to edit package %s') %
                    (str(user), package.id)}

    # Leave the core CKAN auth to work out the hierarchy stuff
    return ckan.logic.auth.delete.package_delete(context, data_dict)

def opsi_extra_fields_editable(context, data_dict):
    """
    Determines whether a dataset's extra-fields are editable directly.

    Typically, this is only something we want sysadmins to be able to do.
    """
    user = context.get('user')
    if dgu_helpers.is_sysadmin_by_context(context):
        return {'success': True}
    else:
        return {'success': False,
                'msg': _('User %s not authorized to edit a dataset\'s extra fields') % str(user)}

def opsi_user_show(context, data_dict):
    user_id = context.get('user','')
    viewing_id = data_dict['id']
    
    if dgu_helpers.is_executive_editor():
        context['keep_email'] = True

    if viewing_id == user_id:
        return {'success': True}

    return opsi_user_list(context, data_dict)

def opsi_user_list(context, data_dict):
    model = context['model']
    user = context.get('user','')
    user_obj = model.User.get(user)

    if dgu_helpers.is_sysadmin_by_context(context):
        return {'success': True}
    
    if dgu_helpers.is_executive_editor(user_obj):
        return {'success': True}

    if not user or not user_obj:
        return {'success': False, 'msg': _('You must be logged in to view the user list')}

    if not len(user_obj.get_groups('organization')):
        return { 'success': False, 'msg': _('Only publishers may view this page') }

    return {'success': True}


def opsi_user_delete(context, data_dict):
    # Sysadmins only
    return {'success': False, 'msg': _('Only sysadmins can delete publishers')}

def dgu_organization_delete(context, data_dict):
    # Sysadmins only
    return { 'success': False, 'msg': _('Only sysadmins can delete publishers') }

def dgu_group_change_state(context, data_dict):
    return dgu_organization_delete(context, data_dict)


def opsi_bulk_publish(context, data_dict):

    model = context['model']
    user = context.get('user')
    user_obj = model.User.get(user)

    if user_obj.sysadmin and 1 == 0:
        return {'success': True}

    # Odgovorni urednik
    if dgu_helpers.is_executive_editor():
        return {'success': True}

    return {
        'success': False,
        'msg': _('User %s not authorized to bulk publish packages') % str(user)
    }
