import os
import shutil
import zipfile
import urllib2
import xml.etree.ElementTree as eTree

EUROVOC_FILENAME = "eurovoc_xml.zip"
GEMET_FILENAME = "gemet-definitions.rdf"

class TagLoader:
    def __init__(self, name, url):
        self._name = name
        self._url = url
        self._vocab = []

    def loader(self):
        response = urllib2.urlopen(self._url)
        data = response.read()
        if self._name == 'eurovoc':
            file_ = open(EUROVOC_FILENAME, 'w')
        else:
            file_ = open(GEMET_FILENAME, 'w')
        file_.write(data)
        file_.close()

        if self._name == 'eurovoc':
            with zipfile.ZipFile(EUROVOC_FILENAME, "r") as zip_ref:
                zip_ref.extractall("eurovoc_xml")

    def parse_xml(self, file_name, to_find):
        with open(file_name, 'r') as f:
            root = eTree.parse(f).getroot()

            namespaces = {
                'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
                'skos': 'http://www.w3.org/2004/02/skos/core#'
            }

            if self._name == 'eurovoc':
                words = root.findall(to_find)
            elif self._name == 'gemet':
                words = root.findall(to_find, namespaces)
            else:
                words = []

            for word in words:
                if len(word.text) < 95:
                    to_add = word.text.replace('"', '*')
                    # if ',' in to_add:
                    #     to_add = '"' + to_add + '"'
                    to_add = {'name': (u' ' + to_add).strip()}
                    if to_add not in self._vocab:
                        self._vocab.append(to_add)

    def clean_up(self):
        if self._name == 'eurovoc':
            os.remove(EUROVOC_FILENAME)
            shutil.rmtree('eurovoc_xml')
        elif self._name == 'gemet':
            os.remove('gemet-definitions.rdf')

    def eurovoc(self):
        self.loader()
        self.parse_xml('eurovoc_xml/thes_sl.xml', 'RECORD/LIBELLE')
        self.parse_xml('eurovoc_xml/dom_sl.xml', 'RECORD/LIBELLE')
        # self.parse_xml('eurovoc_xml/uf_sl.xml', 'RECORD/UF/UF_EL')
        self.parse_xml('eurovoc_xml/desc_sl.xml', 'RECORD/LIBELLE')
        self.clean_up()
        print('\nFinished.')
        return self._vocab

    def gemet(self):
        self.loader()
        self.parse_xml('gemet-definitions.rdf', 'rdf:Description/skos:prefLabel')
        self.clean_up()
        print('\nFinished.')
        return self._vocab

    def get_vocab(self):
        if self._name == 'eurovoc':
            return self.eurovoc()
        elif self._name == 'gemet':
            return self.gemet()
