# -*- coding: utf-8 -*-

import logging
import sys
import os
import re
import sqlalchemy
from datetime import datetime
from ckan.lib.cli import CkanCommand

class DodajOpisePodrocij(CkanCommand):

    """
    """
    summary = __doc__.split('\n')[0]
    usage = __doc__
    max_args = 1
    min_args = 0
    
    """
    Še lažje bi bilo, če bi s paster ukazom naredili vocabulary, tako kot se je sedaj delalo z možnimi vrednostmi metapodatkovnih polj, npr:
    name = "podrocja_opis", tags = [
        {'name': u'Energetika_To je opis področja energetike'},
    ]
    metoda getpodrocje_description() bi nato preskenirala vse tage, primerjala vsebino pred znakom '_' z imenom področja ter izpisalo opis področja, ki sledi znaku '_'
    """
    
    #paster --plugin=ckanext-dgu dodaj_opise_podrocij -c /var/ckan/ckan.ini

    def __init__(self, name):
        super(CkanCommand, self).__init__(name)
        self.opisPodrocij = [ (u"Splošno",u"Tu najdete raznovrstne odprte podatke Slovenije. Če vas zanima specifična tema, izberite eno od področij."),
            (u"Energetika",u"Odkrijte podatke o učinkoviti rabi in obnovljivih virih energije, energetiki in rudarstvu."),
            (u"Finance in davki", u"Tu najdete veliko podatkov s področja podjetništva, tehnološkega razvoja, trgovinske politike, turizma, regionalnega razvoja. Lahko raziščite podatke o finančnem sistemu, javnih financah, davkih, ..."),
            (u"Gospodarstvo",u"Tu najdete veliko podatkov s področja podjetništva, tehnološkega razvoja, trgovinske politike, turizma, regionalnega razvoja. Lahko raziščite podatke o finančnem sistemu, javnih financah, davkih, ..."),
            (u"Izobraževanje. kultura in šport",u"Želite izvedeti več o našem izobraževalnem sistemu, kulturi, kulturni dediščini, umetnosti, športu, ... Ste na pravi strani."),
            (u"Kmetijstvo. ribištvo. gozdarstvo in prehrana",u"Želite izvedeti kaj več o zdravi prehrani, kmetijstvu, ribištvu, lovstvu, gozdarstvu. Še posebej nekaj o veterinarstvu ali pa o kakovosti živil in drugih sorodnih temah. Ste na pravi strani."),
            (u"Lokalna samouprava",u"Raziščite podatke o Vladi RS, javnem sektorju in lokalni samoupravi. Kdo so?  Objavljeni so podatki, aplikacije in druge zanimive informacije.\n Pomembno: Podatki, ki jih lokalna samouprava objavlja so objavljeni po vsebini. Primer: podatki o občinskem proračunu so objavljeni v področju Gospodarstvo, finance, davki."),
            (u"Mednarodne zadeve",u"Poglejte si Slovenijo na mednarodnem političnem, gospodarskem, prosvetnem, kulturnem in drugih področjih."),
            (u"Okolje in prostor",u"Tu najdete podatke, povezane s podnebnimi spremembami, podatke o okolju,  naravi, odpadkih, onesnaževanju, vremenu, vodi, zraku, krajini, gradbeništvu, ..."),
            (u"Pravosodje. pravni sistem in javna varnost",u"Kako deluje naše pravosodje, naša sodišča, kakšen pravni sistem imamo in kaj vse vključuje področje javne varnosti (kriminaliteta, javni red, zasebno varovanje, občinsko redarstvo, orožje, ...)? Podatke o vsem tem najdete tukaj."),
            (u"Prebivalstvo in družba",u"Raziščite podatke o prebivalcih v Sloveniji, o izdanih osebnih dokumentih, društvih, ustanovah, prireditvah, političnih strankah, volitvah v Sloveniji, ..."),
            (u"Promet in infrastruktura",u"Raziščite podatke o cestnem, železniškem, zračnem in pomorskem prometu. Najdite podatke, ki vam bodo olajšali gibanje od točke A do točke B."),
            (u"Sociala in zaposlovanje",u"Tukaj najdete podatke o našem zdravju, izvajanjem zdravstvene dejavnosti, sociali, na razpolago so podatki s področja  zaposlovanja, ..."),
            (u"Javni sektor",u"Raziščite podatke o Vladi RS, javnem sektorju in lokalni samoupravi. Kdo so?  Objavljeni so podatki, aplikacije in druge zanimive informacije.\n Pomembno: Podatki, ki jih lokalna samouprava objavlja so objavljeni po vsebini. Primer: podatki o občinskem proračunu so objavljeni v področju Gospodarstvo, finance, davki."),
            (u"Zdravje",u"Tukaj najdete podatke o našem zdravju, izvajanjem zdravstvene dejavnosti, sociali, na razpolago so podatki s področja  zaposlovanja, ..."),
            (u"Znanost in tehnologija",u"Raziščite podatke s področja znanosti in tehnologije.")
        ]

    def command(self):
        self._load_config()
        log = logging.getLogger(__name__)
        import ckan.plugins.toolkit as tk
        import ckan.model as model
        
        #http://docs.sqlalchemy.org/en/latest/core/metadata.html
        from pylons import config
        from sqlalchemy import *
        from sqlalchemy.engine import reflection
        
        #Do we reset the database?
        reset = False
        if len(self.args) == 1:
            if self.args[0]=='reset':
                reset = True
            else:
                log.info("Got strange args: " + str(self.args))
                log.info("Either use no arguments to create podrocje_description_table if it does not exist or add non-existing rows into it;")
                log.info("or use the 'reset' command line argument to totally drop the existing database and create a new one.")
                sys.exit(1)
        elif len(self.args) > 1:
            log.info("Got strange args: " + str(self.args))
            log.info("Either use no arguments to create podrocje_description_table if it does not exist or add non-existing rows into it;")
            log.info("or use the 'reset' command line argument to totally drop the existing database and create a new one.")
        else:
            reset = False
        
        # for postgresql we want to enforce utf-8
        sqlalchemy_url = config.get('sqlalchemy.url', '')
        if sqlalchemy_url.startswith('postgresql://'):
            extras = {'client_encoding': 'utf8'}
        else:
            extras = {}
    
        engine = sqlalchemy.engine_from_config(config, 'sqlalchemy.', **extras)
        metadata = MetaData()
        
        #We don't use sessions
        # Construct a sessionmaker factory object
        #session = sessionmaker()
        # Bind the sessionmaker to engine
        #session.configure(bind=engine)
        # Generate a session to work with
        #s = session()
        
        #Define the table
        podrocje_description_table = Table('podrocje_description_table', metadata,
            Column('podrocje_name', String(512), primary_key=True),
            Column('podrocje_description', String(2048), nullable=False)
        )
        
        if reset == True:
            print "Reseting the podrocje_description_table - dropping it and recreating."
            #Drop the table and recreate it
            #http://stackoverflow.com/questions/19175311/how-to-create-only-one-table-with-sqlalchemy
            #podrocje_description_table.create(engine)
            
            #Check if the table already exists
            inspector = reflection.Inspector.from_engine(engine)
            if "podrocje_description_table" in inspector.get_table_names():
                #Table "podrocje_description_table" exists
                #drop it and in its place create an updated table
                print "table exists, drop it and then recreate it"
                
                podrocje_description_table.drop(engine, checkfirst=True)
                podrocje_description_table.create(engine, checkfirst=True)
                
                #Fill the table with data!
                #http://stackoverflow.com/questions/13204262/insert-data-into-specific-table-sqlalchemy
                for podrocje_opis in self.opisPodrocij:
                    engine.execute(podrocje_description_table.insert(), podrocje_name=podrocje_opis[0], podrocje_description=podrocje_opis[1])
    
            else:
                #Table "podrocje_description_table" does not exist
                #Create it
                print "table does not exist, create it"
    
                podrocje_description_table.create(engine, checkfirst=True)
    
                #Fill the table with data!
                #http://stackoverflow.com/questions/13204262/insert-data-into-specific-table-sqlalchemy
                for podrocje_opis in self.opisPodrocij:
                    engine.execute(podrocje_description_table.insert(), podrocje_name=podrocje_opis[0], podrocje_description=podrocje_opis[1])
            
        else:
            #Create the database or update it with missing rows
            print "Checking if podrocje_description_table exists"
            
            #Check if the table already exists
            inspector = reflection.Inspector.from_engine(engine)
            if "podrocje_description_table" in inspector.get_table_names():
                print "podrocje_description_table exists, updating it (not touching existing rows)"
                
                #Get the contents of the table
                selectStatement = select([podrocje_description_table])
                resultStatement = engine.execute(selectStatement)
                
                resultPodrocjeName = []
                for podrocje_result in resultStatement:
                    resultPodrocjeName.append(podrocje_result[0])
                
                for podrocje_opis in self.opisPodrocij:
                    if podrocje_opis[0] not in resultPodrocjeName:
                        engine.execute(podrocje_description_table.insert(), podrocje_name=podrocje_opis[0], podrocje_description=podrocje_opis[1])
                
            else:
                print "podrocje_description_table does not exist, creating it"
                podrocje_description_table.create(engine, checkfirst=True)
    
                #Fill the table with data!
                #http://stackoverflow.com/questions/13204262/insert-data-into-specific-table-sqlalchemy
                for podrocje_opis in self.opisPodrocij:
                    engine.execute(podrocje_description_table.insert(), podrocje_name=podrocje_opis[0], podrocje_description=podrocje_opis[1])
            
        print "table podrocje_description_table now contains:"
        selectStatement = select([podrocje_description_table])
        resultStatement = engine.execute(selectStatement)
        for row in resultStatement:
            print(row)
        
        #http://www.rmunn.com/sqlalchemy-tutorial/tutorial-0.1.html
        #db = sqlalchemy.engine_from_config(config, 'sqlalchemy.', **extras)
        #podrocje_description_table = Table('podrocje_description_table', db, autoload=True)
        #selectStatement = podrocje_description_table.select(podrocje_description_table.podrocje_name == podrocje_name)
        #rs = selectStatement.execute()
        #for row in rs:
        #    print row
        
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        """
        opisPodrocij_tags = [ {'name': u'Energetika---opis energetike'},
            {'name': u'Finance in davki---opis davkov'},
            {'name': u'Gospodarstvo---opis gospodarstva'},
            {'name': u'Izobraževanje. kultura in šport---opis izobraževanja. kulture ter športa'},
            {'name': u'Kmetijstvo. ribištvo. gozdarstvo in prehrana---opis kmetijstva. ribištva ter gozdarstva'},
            {'name': u'Lokalna samouprava---opis lokalne samouprave'},
            {'name': u'Mednarodne zadeve---opis mednarodnih zadev'},
            {'name': u'Okolje in prostor---opis okolja in prostora'},
            {'name': u'Pravosodje. pravni sistem in javna varnost---opis pravosodja. pravnih sistemov in javne varnosti'},
            {'name': u'Prebivalstvo in družba---opis prebivalstva in družbe'},
            {'name': u'Promet in infrastruktura---opis prometa in infrastrukture'},
            {'name': u'Sociala in zaposlovanje---opis sociale in zaposlovanja'},
            {'name': u'Javni sektor---opis vlade in javnega sektorja'},
            {'name': u'Zdravje---opis zdravja'},
            {'name': u'Znanost in tehnologija---opis znanosti in tehnologije'}
        ]
        
        existing = model.Vocabulary.by_name('opis_podrocja')
        if not existing:
            opisPodrocij_dict={'name': 'opis_podrocja', 'tags': opisPodrocij_tags}
            
            print "Running vocabulary_create"
            result1 = tk.get_action('vocabulary_create')(data_dict = opisPodrocij_dict)
            print result1
            
        else:
            vocabularyPodrocjeId=""
            try:
                podrocjeValues = tk.get_action('vocabulary_list')()
                for i in range(len(podrocjeValues)):
                    if podrocjeValues[i]['name'] == 'opis_podrocja':
                        vocabularyPodrocjeId = podrocjeValues[i]["id"]
                        break
            except tk.ObjectNotFound:
                print "Napaka pri izvajanju vocabulary_update"
                print tk.ObjectNotFound
            
            if vocabularyPodrocjeId == "":
                print "Failure to update the vocabulary!"
                return
            
            opisPodrocij_dict={'id': vocabularyPodrocjeId, 'tags': opisPodrocij_tags}
            
            print "Running vocabulary_update"
            result1 = tk.get_action('vocabulary_update')(data_dict = opisPodrocij_dict)
            print result1
        """