# -*- coding: utf-8 -*-

import logging
from pylons import config
from ckan import logic
import ckan.model as model
import ckan.plugins.toolkit as tk
from ckan.lib.cli import CkanCommand
from ckanext.dgu.bin.tag_loader import TagLoader
from ckan.lib.navl.validators import not_empty, not_missing


class ExternalTags(CkanCommand):
    """
    """
    summary = __doc__.split('\n')[0]
    usage = __doc__
    max_args = 2
    min_args = 0

    def __init__(self, name):
        super(CkanCommand, self).__init__(name)
        self._reset_tags = False

    def helper(self, name, vocab):
        existing = model.Vocabulary.by_name(name)

        if not existing:
            vocabulary_schema = logic.schema.default_create_vocabulary_schema()
        else:
            vocabulary_schema = logic.schema.default_update_vocabulary_schema()

        tag_schema = logic.schema.default_tags_schema()
        tag_schema['name'] = [not_missing, not_empty, unicode]
        vocabulary_schema['tags'] = tag_schema

        context = {
            'model': model,
            'schema': vocabulary_schema
            }

        if not existing:
            result1 = tk.get_action('vocabulary_create')(
                context=context,
                data_dict={'name': name, 'tags': vocab})
            print result1

        elif self._reset_tags:
            result1 = tk.get_action('vocabulary_update')(
                context=context,
                data_dict={'id': existing.id, 'tags': vocab})
            print result1

    def command(self):
        self._load_config()
        log = logging.getLogger(__name__)

        source_arg = 0
        self._reset_tags = len(self.args) > 0 and self.args[0] == 'reset'
        if self._reset_tags:
            source_arg = 1

        if len(self.args) > source_arg:
            url = 'ckan.' + self.args[source_arg] + '_url'
            vocab = TagLoader(
                self.args[source_arg], config.get(url)
            ).get_vocab()

            self.helper(self.args[source_arg], vocab)
        else:
            eurovoc = TagLoader('eurovoc', config.get('ckan.eurovoc_url')).get_vocab()
            gemet = TagLoader('gemet', config.get('ckan.gemet_url')).get_vocab()

            self.helper('eurovoc', eurovoc)
            self.helper('gemet', gemet)



# eurovoc_url = 'http://publications.europa.eu/mdr/resource/thesaurus/eurovoc-20160630-0/xml/eurovoc_xml.zip'
# gemet_url = 'http://www.eionet.europa.eu/gemet/exports/latest/sl/gemet-definitions.rdf'
