# -*- coding: utf-8 -*-

import logging
import json
import os
from ckan.lib.cli import CkanCommand

log = logging.getLogger('ckanext')


class InsertStaticPages(CkanCommand):
    summary = "Inserts default OPSI static pages to the database. " \
              "Pages' data is located in JSON file `pages.json`. " \
              "Those pages can later be edited on OPSI platform. " \
              "If page with a given name already exists, it is skipped."
    def __init__(self, name):
        super(CkanCommand, self).__init__(name)
        f = open(str(os.path.split(os.path.realpath(__file__))[0]) + "/pages.json",'r')
        json_buf = f.read()
        self.static_pages = json.loads(json_buf)

    def command(self):
        self._load_config()

        import ckan.model as model
        import ckan.plugins.toolkit as tk
        from ckanext.pages import db

        model.Session.remove()
        model.Session.configure(bind=model.meta.engine)

        context = {}
        context['user'] = 'frontend'
        data_dict = {}

        current_static_pages_list = tk.get_action('ckanext_pages_list')(context=context, data_dict=data_dict)
        try:
            for page in self.static_pages:
                exists = False
                for currPage in current_static_pages_list:
                    if page["name"] == currPage['name']:
                        log.info("Static page %s exists" % page["name"])
                        exists = True
                if not exists:
                    if db.Page is None:
                        db.init_db(context['model'])
                    new_page = db.Page(name=page['name'])
                    setattr(new_page, 'title', page['title'])
                    setattr(new_page, 'content', page['content'])
                    setattr(new_page, 'publish_date', '2023-05-17T00:00:00')
                    setattr(new_page, 'page_type', 'static')
                    setattr(new_page, 'private', False)
                    setattr(new_page, 'group_id', None)
                    model.Session.add(new_page)
                    model.Session.commit()
                    log.info("Inserted static page %s" % page["name"])
            model.Session.close()
        except Exception, e:
            log.info("insert static page exception: %r" % e)
