# -*- coding: utf-8 -*-

import logging
from ckan.lib.cli import CkanCommand
from ckan.model.meta import metadata, mapper, Session

class Migrate(CkanCommand):
    """
    Migration tasks when upgrading to specific version
    """
    summary = __doc__.split('\n')[0]
    usage = __doc__
    max_args = 0
    min_args = 0
    #paster --plugin=ckanext-dgu migrate -c /var/ckan/ckan.ini

    def __init__(self, name):
        super(CkanCommand, self).__init__(name)

    def command(self):
        self._load_config()
        log = logging.getLogger(__name__)
        self.migrate_v2_10()
        self.migrate_v2_20()

    def migrate_v2_10(self):
        from ckanext.dgu.model.package_likes import migrate_v2_10 as migrate_likes
        migrate_likes()

    def migrate_v2_20(self):
        log = logging.getLogger(__name__)
        #create additional index to aid optimized revision retrieval which is slower when ordering by revision_timestamp
        log.info('Migrating v2.20 - adding index for package_extra with better column ordering')
        conn = Session.connection()

        statements = '''
        CREATE INDEX idx_package_extra_package_period on package_extra_revision(package_id, revision_timestamp, expired_timestamp);
        '''
        conn.execute(statements)

        Session.commit()
        log.info('Migrated to v2.20')
