# -*- coding: utf-8 -*-
import logging
from pylons import config
from ckan.lib.cli import CkanCommand
# No other CKAN imports allowed until _load_config is run,
# or logging is disabled

class InitDB(CkanCommand):
    """
    """
    summary = __doc__.split('\n')[0]
    usage = __doc__
    max_args = 0
    min_args = 0

    OPSI_ROLES = [{'name':'glavni_uredniki', 'title':u'Glavni uredniki'},
                {'name':'podrocni_uredniki', 'title':u'Področni uredniki'},
                {'name':'vsebinski_uredniki', 'title':u'Vsebinski uredniki'},
                #{'name':'tehnicni_uredniki', 'title':u'Tehnični uredniki'},
                {'name':'sistemski_administratorji', 'title':u'Sistemski administratorji'}]

    def __init__(self, name):
        super(InitDB, self).__init__(name)

    def command(self):
        """
        """
        self._load_config()
        log = logging.getLogger(__name__)

        import ckan.model as model
        import ckan.plugins.toolkit as tk

        context = {}
        context['user'] = 'frontend'
        data_dict = {}
        data_dict['groups'] = [g['name'] for g in self.OPSI_ROLES]
        current_group_list = tk.get_action('group_list')(context=context, data_dict=data_dict)
        needs_init = False
        for g in self.OPSI_ROLES:
            if g['name'] not in current_group_list:
                needs_init = True

        if not needs_init:
            log.info("OPSI roles are already initialized!")
            return

        model.Session.remove()
        model.Session.configure(bind=model.meta.engine)
        log.info("Database access initialised")

        rev = model.repo.new_revision()
        rev.author = u'vs_admin'
        rev.message = u'Initializing roles for OPSI as CKAN groups'

        for g in self.OPSI_ROLES:
            if g['name'] not in current_group_list:
                new_group = model.Group(name=g['name'])
                setattr(new_group, 'title', g['title'])
                setattr(new_group, 'type', 'VS_role')
                model.Session.add(new_group)
        model.Session.commit()
        model.Session.close()

        log.debug("OPSI roles have been initialized")