# -*- coding: utf-8 -*-

import ckan.plugins.toolkit as t
import ckan.lib.helpers as h
import ckan.logic as logic
import ckan.model as model
import ckan.new_authz as new_authz
from ckan.lib.base import BaseController, abort, render
from ckan.controllers.user import UserController as CoreUserController
from ckan.common import _, request
from ckanext.dgu.lib.user_helpers import merge_users_on_delete, move_user
from ckanext.dgu.lib.reports import refresh_opsi_reports_async

c = t.c
get_action = logic.get_action
check_access = logic.check_access
NotFound = logic.NotFound
NotAuthorized = logic.NotAuthorized

import logging
log = logging.getLogger(__name__)


class UserController(CoreUserController):
    def me(self, locale=None):
        if not c.user:
            h.redirect_to(locale=locale, controller='user', action='login', id=None)
        user_ref = c.userobj.get_reference_preferred_for_uri()
        # redirect to the user-read instead of user-dashboard
        h.redirect_to(locale=locale, controller='user', action='read', id=user_ref)

    def read(self, id=None):
        context = {'model': model, 'session': model.Session,
                   'user': c.user or c.author, 'auth_user_obj': c.userobj,
                   'for_view': True}
        data_dict = {'id': id,
                     'user_obj': c.userobj}

        context['with_related'] = True

        c.is_sysadmin = new_authz.is_sysadmin(c.user)
        try:
            user_dict = get_action('user_show_no_limit')(context, data_dict)
        except NotFound:
            abort(404, _('User not found'))
        except NotAuthorized:
            abort(401, _('Not authorized to see this page'))
        c.user_dict = user_dict
        c.is_myself = user_dict['name'] == c.user
        c.about_formatted = h.render_markdown(user_dict['about'])

        # The legacy templates have the user's activity stream on the user
        # profile page, new templates do not.
        #Disabled for DGU
        #if h.asbool(config.get('ckan.legacy_templates', False)):
        #    c.user_activity_stream = get_action('user_activity_list_html')(
        #        context, {'id': c.user_dict['id']})

        return render('user/read.html')
    
    # overrides the ckan implementation
    def index(self):
        c.q = request.params.get('q', '')
        c.order_by = request.params.get('order_by', 'name')

        context = {'return_query': True, 'user': c.user or c.author,
                   'auth_user_obj': c.userobj}

        data_dict = {'q': c.q,
                     'order_by': c.order_by}
        try:
            check_access('user_list', context, data_dict)
        except NotAuthorized:
            abort(401, _('Not authorized to see this page'))

        users_list = get_action('user_list')(context, data_dict)
        c.users = users_list

        return render('user/list.html')

    def delete(self, id):
        from ckan import model
        errors = {}
        context = {'model': model, 'session': model.Session,
                   'user': c.user or c.author, 'auth_user_obj': c.userobj,
                   'for_view': True}
        data_dict = {'id': id,
                     'user_obj': c.userobj}
        context['with_related'] = False

        try:
            check_access('user_delete', context, data_dict)
        except NotAuthorized:
            abort(401, _('Not authorized to see this page'))

        c.is_sysadmin = new_authz.is_sysadmin(c.user)

        try:
            user_dict = get_action('user_show')(context, data_dict)
        except NotFound:
            abort(404, _('User not found'))
        except NotAuthorized:
            abort(401, _('Not authorized to see this page'))
        c.user_dict = user_dict
        c.is_myself = user_dict['name'] == c.user

        if request.POST:
            if 'cancel' in request.params:
                h.redirect_to(h.url_for(controller='user', action='read', id=id))
                return
            else:
                if c.is_myself:
                    errors['user'] = "Uporabnika ni dovoljeno izbrisati."
                new_user_id = request.params.get('creator_user_id', None)
                if c.user_dict['number_administered_packages'] > 0:
                    if not new_user_id:
                        errors[_('Editor')] = _("This field is required")
                    if new_user_id == c.user_dict['id'] or new_user_id == c.user_dict['name']:
                        errors[_('Editor')] = u"Napačen izbor urednika (%s)" % c.user_dict['display_name']

                if not errors:

                    if c.user_dict['number_administered_packages'] > 0:
                        merged_users = merge_users_on_delete(c.user_dict['id'], new_user_id, context)
                        _error = merged_users.get('error', None)
                    else:
                        _error = None

                    if _error:
                        errors[_('Editor')] = _error
                    else:
                        try:
                            model = context['model']
                            user_id = logic.get_or_bust(data_dict, 'id')
                            user = model.User.get(user_id)

                            if user is None:
                                raise NotFound('Uporabnik "{id}" ni bil najden.'.format(id=user_id))

                            user.delete()
                            model.repo.commit()

                            user_index = h.url_for(controller='ckanext.dgu.controllers.user:UserController', action='index')
                            h.flash_success('Uporabnik je bil izbrisan.')
                            h.redirect_to(user_index)
                        except NotAuthorized:
                            msg = _('Unauthorized to delete user with id "{user_id}".')
                            abort(401, msg.format(user_id=id))

        c.errors = errors

        return render('user/delete.html')

