import ckan.logic as logic
from ckan import plugins
from ckan.lib.base import h
import ckan.lib.uploader as uploader
from ckanext.dgu.lib.helpers import tidy_up_package
_check_access = logic.check_access
_get_action = logic.get_action
ValidationError = logic.ValidationError
_get_or_bust = logic.get_or_bust

import logging
log = logging.getLogger(__name__)
import magic
import os
uploadable_mimetypes = ['application/vnd.ogc.wms_xml',
                        'application/vnd.google-earth.kml+xml',
                        'application/x-netcdf',
                        'application/acad', 
                        'application/x-acad',
                        'application/autocad_dwg',
                        'image/x-dwg',
                        'application/dwg',
                        'application/x-dwg',
                        'application/x-autocad',
                        'image/vnd.dwg',
                        'drawing/dwg',
                        'application/dxf',
                        'application/x-autocad',
                        'application/x-dxf',
                        'drawing/x-dxf',
                        'image/vnd.dxf',
                        'image/x-autocad',
                        'image/x-dxf',
                        'zz-application/zz-winassoc-dxf',
                        'application/vnd.geo+json',
                        'application/gml+xml',
                        'image/x-mrsid-image',
                        'application/x-mrsid',
                        'image/svg+xml',
                        'image/xyz',
                        'image/jpg',
                        'image/jpeg',
                        'image/gif',
                        'image/tiff',
                        'image/geotiff',
                        'image/geotiffint16',
                        'image/png',
                        'application/json',
                        'text/x-json',
                        'text/xml',
                        'application/xml',
                        'text/rss+xml',
                        'application/rdf+xml',
                        'text/turtle',
                        'application/sparql-results+xml',
                        'application/sparql-query',
                        'text/csv',
                        'text/comma-separated-values',
                        'text/psv',
                        'text/pipe-separated-values',
                        'text/plain',
                        'application/pdf',
                        'application/rtf',
                        'application/msword',
                        'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                        'application/vnd.ms-word.document.macroEnabled.12',
                        'application/vnd.oasis.opendocument.text',
                        'application/x-vnd.oasis.opendocument.text',
                        'application/mspowerpoint',
                        'application/vnd.ms-powerpoint.presentation.macroEnabled.12',
                        'application/vnd.ms-powerpoint',
                        'application/vnd.oasis.opendocument.presentation',
                        'application/x-vnd.oasis.opendocument.presentation',
                        'application/vnd.oasis.opendocument.spreadsheet',
                        'application/excel',
                        'application/x-excel',
                        'application/x-msexcel',
                        'application/vnd.ms-excel',
                        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                        'application/vnd.ms-excel.sheet.binary.macroenabled.12',
                        'application/vnd.ms-excel.sheet.macroenabled.12',
                        'application/vnd.msexcel',
                        'application/x-zip',
                        'application/x-compressed',
                        'application/x-zip-compressed',
                        'application/zip',
                        'multipart/x-zip',
                        'application/x-gzip',
                        'application/x-bittorrent',
                        'application/octet-stream'
                        ]
def validate_mimetype(file):
    mime = magic.from_buffer(file.read(24), mime=True)
    file.seek(0, os.SEEK_SET)
    if mime in uploadable_mimetypes:
        return True
    log.debug("Denied uploading of mime type: "+str(mime))
    return False

def opsi_action_resource_create(context, data_dict):
    '''Appends a new resource to a datasets list of resources.

    :param package_id: id of package that the resource needs should be added to.
    :type package_id: string
    :param url: url of resource
    :type url: string
    :param revision_id: (optional)
    :type revisiion_id: string
    :param description: (optional)
    :type description: string
    :param format: (optional)
    :type format: string
    :param hash: (optional)
    :type hash: string
    :param name: (optional)
    :type name: string
    :param resource_type: (optional)
    :type resource_type: string
    :param mimetype: (optional)
    :type mimetype: string
    :param mimetype_inner: (optional)
    :type mimetype_inner: string
    :param webstore_url: (optional)
    :type webstore_url: string
    :param cache_url: (optional)
    :type cache_url: string
    :param size: (optional)
    :type size: int
    :param created: (optional)
    :type created: iso date string
    :param last_modified: (optional)
    :type last_modified: iso date string
    :param cache_last_updated: (optional)
    :type cache_last_updated: iso date string
    :param webstore_last_updated: (optional)
    :type webstore_last_updated: iso date string
    :param upload: (optional)
    :type upload: FieldStorage (optional) needs multipart/form-data

    :returns: the newly created resource
    :rtype: dictionary

    '''
    import ntpath
    model = context['model']
    user = context['user']

    package_id = _get_or_bust(data_dict, 'package_id')
    # The only line removed from ckan(release-v2.2-dgu): data_dict.pop('package_id')

    pkg_dict = _get_action('package_show')(
        dict(context, return_type='dict', for_update=True),
        {'id': package_id})

    _check_access('resource_create', context, data_dict)
    
    #OPSI edit: check access appends package and its id to data_dict which may cause incorrect generation of resource filenames, we remove it
    if data_dict.get('package', ''):
        del data_dict['package']
    if data_dict.get('id', ''):
        del data_dict['id']
    
    #remove date if it is empty (should be saved as individual resource)
    if data_dict.get('date', 'not existing') == '':
        del data_dict['date']

    #if description is missing, but name is specified, copy it
    if not data_dict.get('description') and data_dict.get('name'):
        data_dict['description'] = data_dict.get('name')

    #pogostost_osvezevanja is override of package field and we remove it from resource
    if data_dict.has_key('pogostost_osvezevanja'):
        if data_dict.get('pogostost_osvezevanja') != '':
            pkg_dict['pogostost_osvezevanja'] = data_dict['pogostost_osvezevanja']
        del data_dict['pogostost_osvezevanja']

    if not 'resources' in pkg_dict:
        pkg_dict['resources'] = []

    if data_dict.get('resource_type') == u'documentation':
        if not 'additional_resources' in pkg_dict:
            pkg_dict['additional_resources'] = []
        pkg_dict['additional_resources'].append(data_dict)
    elif 'date' in data_dict:
        if not 'timeseries_resources' in pkg_dict:
            pkg_dict['timeseries_resources'] = []
        pkg_dict['timeseries_resources'].append(data_dict)
    else:
        if not 'individual_resources' in pkg_dict:
            pkg_dict['individual_resources'] = []
        pkg_dict['individual_resources'].append(data_dict)

    #remove dir paths from Internet explorer
    uploadedFile = data_dict.get('upload', None)
    if hasattr(uploadedFile, 'filename'):
        uploadedFile.filename = ntpath.basename(uploadedFile.filename)
    
    upload = uploader.ResourceUpload(data_dict)

    # TODO: currently only the filename is stored in the URL field of uploaded files. It would be beneficial to find a
    # way to generate a full URL here (e.g. by generating an ID for the resource beforehand) to avoid changing between
    # the filename and a full URL later on after archiver processing etc.

    # Gather IDs of existing resources to compare after when getting the new resource ID for the uploader.
    # The upstream implementation just uses the last resource but it seems that it is not guaranteed that the
    # new resource is returned as the last (observed when integrating the downloadall plugin which could
    # overwrite other resource files in some cases) - an uploaded "documentation" type was usually put first.
    old_resource_ids = []
    for resource in pkg_dict['resources']:
        old_resource_ids.append(resource['id'])

    pkg_dict['resources'].append(data_dict)

    try:
        context['defer_commit'] = True
        context['use_cache'] = False
        context['ignore_auth'] = True #content editor will otherwise not be able to upload to active datasets
        tidy_up_package(pkg_dict)
        _get_action('package_update')(context, pkg_dict)
        context.pop('defer_commit')
    except ValidationError, e:
        try:
            errors = e.error_dict['resources'][-1]
        except KeyError:
            errors = e.error_dict
        raise ValidationError(errors)

    if hasattr(upload, 'filename') and upload.filename and upload.filename.endswith('.exe'):
        raise ValidationError({'upload': ['Datotek formata EXE ni dovoljeno nalagati na portal OPSI.']})

    if hasattr(upload, 'upload_file') and not validate_mimetype(upload.upload_file):
        mime = magic.from_buffer(upload.upload_file.read(24), mime=True)
        upload.upload_file.seek(0, os.SEEK_SET)
        raise ValidationError({'upload': ['Datotek formata '+mime+' ni dovoljeno nalagati na portal OPSI.']})

    new_resource_id = context['package'].resources[-1].id #fallback to upstream system of just using last resource
    for resource in context['package'].resources:
        if resource.id not in old_resource_ids:
            new_resource_id = resource.id

    ## Get out resource_id resource from model as it will not appear in
    ## package_show until after commit
    resource_size_limit = context.get('max_resource_size', 16*1024*1024*1024) if context.get('is_local') else uploader.get_max_resource_size()
    upload.upload(new_resource_id, resource_size_limit)

    model.repo.commit()

    ##  Run resource show get out actual last resource
    resource = _get_action('resource_show')(context, {'id': new_resource_id})

    for plugin in plugins.PluginImplementations(plugins.IResourceController):
        plugin.after_create(context, resource)

    for plugin in plugins.PluginImplementations(plugins.IResourceController):
        plugin.after_create(context, resource)

    return resource


def opsi_datapusher_submit(context, data_dict):
    ''' Extension of built-in datapusher_submit to decrease the processing time
    by not retrieving all user data since only the api key is needed
    '''
    context['return_minimal'] = True
    return logic.get_action('datapusher_submit')(context, data_dict)
