import ckan.logic
import ckan.logic.action
import ckan.plugins as plugins
from ckan.logic import get_or_bust, get_action

ValidationError = ckan.logic.ValidationError
NotFound = ckan.logic.NotFound
_check_access = ckan.logic.check_access
_get_or_bust = ckan.logic.get_or_bust
_get_action = ckan.logic.get_action


def opsi_action_resource_delete(context, data_dict):
    '''Delete a resource from a dataset.

    You must be a sysadmin or the owner of the resource to delete it.

    :param id: the id of the resource
    :type id: string

    '''
    model = context['model']
    id = _get_or_bust(data_dict, 'id')

    entity = model.Resource.get(id)

    if entity is None:
        raise NotFound

    _check_access('resource_delete',context, data_dict)

    package_id = entity.get_package_id()

    pkg_dict = _get_action('package_show')(dict(context, for_update=True), {'id': package_id})

    if pkg_dict.get('resources'):
        pkg_dict['resources'] = [r for r in pkg_dict['resources'] if not
        r['id'] == id]

    # resource has to be deleted from all resource lists,
    # otherwise it is merged back to the list of resources
    if pkg_dict.get('individual_resources'):
        pkg_dict['individual_resources'] = [r for r in pkg_dict['resources'] if not
        r['id'] == id and r['resource_type'] == 'file']
    if pkg_dict.get('timeseries_resources'):
        pkg_dict['timeseries_resources'] = [r for r in pkg_dict['resources'] if not
        r['id'] == id and r['resource_type'] == 'file']
    if pkg_dict.get('additional_resources'):
        pkg_dict['additional_resources'] = [r for r in pkg_dict['resources'] if not
        r['id'] == id and r['resource_type'] == 'documentation']

    try:
        context['ignore_auth'] = True #content editor will otherwise not be able to delete from active datasets
        pkg_dict = _get_action('package_update')(context, pkg_dict)
    except ValidationError, e:
        if 'resources' not in e.error_dict:
            raise Exception('resource_delete error: %r' %
                            e.error_dict)

        errors = e.error_dict['resources'][-1]
        raise ValidationError(errors)

    model.repo.commit()
