import ckan.logic as logic
from ckan.logic import get_or_bust, get_action
from ckan.logic import NotFound, check_access
from ckan.logic import side_effect_free
import ckan.lib.dictization.model_dictize as model_dictize
from ckan import plugins
import ckan.lib.plugins as lib_plugins
from ckan.lib.navl.dictization_functions import validate, missing
from ckan.logic.action.get import organization_show
from ckanext.dgu.model.schema_codelist import Schema, Codelist
from ckanext.dgu.lib.helpers import is_executive_editor
import ckan.model.misc as misc
#from ckan.plugins.toolkit as t
import sqlalchemy
_or_ = sqlalchemy.or_
log = __import__('logging').getLogger(__name__)
_check_access = logic.check_access
_get_or_bust = logic.get_or_bust

@side_effect_free
def package_show(context, data_dict):
    from ckan.logic.action.get import package_show as ckan_package_show
    pkg_dict = ckan_package_show(context, data_dict)
    #quick fix since it does not seem to work by updating schema with value_if_missing
#    if not pkg_dict.has_key('municipal'):
#        pkg_dict['municipal'] = False
    if pkg_dict.has_key('tag_string') and pkg_dict.get('tag_string') != missing:
        return pkg_dict
    tag_string = ''
    for tag in pkg_dict.get('tags', []):
        if tag_string != '':
            tag_string = tag_string + ', '+tag['name']
        else:
            tag_string = tag['name']
    pkg_dict['tag_string'] = tag_string
    return pkg_dict

@side_effect_free
def publisher_show(context, data_dict):
    '''Shows publisher details.
    Based on group_show, but has parent group, as well as the child groups.

    May raise NotFound or NotAuthorized.
    '''
    group_dict = organization_show(context, data_dict)

    model = context['model']
    id = get_or_bust(data_dict, 'id')
    group = model.Group.get(id)

    parent_groups = group.get_parent_groups(type='organization')
    group_dict['parent_group'] = {'id': parent_groups[0].id, 'name': parent_groups[0].name} \
                                 if parent_groups else None

    return group_dict

def user_show_no_limit(context, data_dict):
    '''Return a user account with all its packages

    Either the ``id`` or the ``user_obj`` parameter must be given.

    :param id: the id or name of the user (optional)
    :type id: string
    :param user_obj: the user dictionary of the user (optional)
    :type user_obj: user dictionary

    :rtype: dictionary

    '''
    model = context['model']

    id = data_dict.get('id',None)
    provided_user = data_dict.get('user_obj',None)
    if id:
        user_obj = model.User.get(id)
        context['user_obj'] = user_obj
        if user_obj is None:
            raise NotFound
    elif provided_user:
        context['user_obj'] = user_obj = provided_user
    else:
        raise NotFound

    check_access('user_show',context, data_dict)

    user_dict = model_dictize.user_dictize(user_obj,context)

    if context.get('return_minimal'):
        return user_dict

    #not needed on OPSI:
#    revisions_q = model.Session.query(model.Revision
#            ).filter_by(author=user_obj.name)
#
#    revisions_list = []
#    for revision in revisions_q.limit(20).all():
#        revision_dict = logic.get_action('revision_show')(context,{'id':revision.id})
#        revision_dict['state'] = revision.state
#        revisions_list.append(revision_dict)
#    user_dict['activity'] = revisions_list

    user_dict['datasets'] = []
    dataset_q = model.Session.query(model.Package).join(model.PackageRole
            ).filter_by(user=user_obj, role=model.Role.ADMIN)

    for dataset in dataset_q:
        try:
            dataset_dict = logic.get_action('package_show')(context, {'id': dataset.id})
        except logic.NotAuthorized:
            continue
        user_dict['datasets'].append(dataset_dict)
        
    #not needed on OPSI:
#    user_dict['num_followers'] = logic.get_action('user_follower_count')(
#            {'model': model, 'session': model.Session},
#            {'id': user_dict['id']})

    return user_dict

@logic.validate(logic.schema.default_autocomplete_schema)
def package_autocomplete(context, data_dict):
    '''Return a list of datasets (packages) that match a string.

    Datasets with names or titles that contain the query string will be
    returned.

    :param q: the string to search for
    :type q: string
    :param limit: the maximum number of resource formats to return (optional,
        default: 10)
    :type limit: int

    :rtype: list of dictionaries

    '''
    model = context['model']

    check_access('package_autocomplete', context, data_dict)

    limit = data_dict.get('limit', 100)
    q = data_dict['q']

    like_q = u"%%%s%%" % q

    query = model.Session.query(model.PackageRevision)
    query = query.filter(model.PackageRevision.state=='active')
    query = query.filter(model.PackageRevision.type=='dataset')
    query = query.filter(model.PackageRevision.current==True)
    query = query.filter(_or_(model.PackageRevision.name.ilike(like_q),
                                model.PackageRevision.title.ilike(like_q)))
    query = query.limit(limit)

    q_lower = q.lower()
    pkg_list = []
    for package in query:
        if package.name.startswith(q_lower):
            match_field = 'name'
            match_displayed = package.name
        else:
            match_field = 'title'
            match_displayed = '%s (%s)' % (package.title, package.name)
        result_dict = {'name':package.name, 'title':package.title,
                       'match_field':match_field, 'match_displayed':match_displayed}
        pkg_list.append(result_dict)

    return pkg_list

@side_effect_free
def schema_list(context, data_dict):
    check_access('schema_list', context, data_dict)

    model = context['model']
    items = model.Session.query(Schema).order_by('title')
    return [item.as_dict() for item in items.all()]

@side_effect_free
def codelist_list(context, data_dict):
    check_access('codelist_list', context, data_dict)

    model = context['model']
    items = model.Session.query(Codelist).order_by('title')
    return [item.as_dict() for item in items.all()]


def tag_autocomplete_all(context, data_dict):
    _check_access('tag_autocomplete', context, data_dict)
    matching_tags, count = _tag_search_all(context, data_dict)
    if matching_tags:
        return [tag.name for tag in matching_tags]
    else:
        return []


def _tag_search_all(context, data_dict):
    model = context['model']

    terms = data_dict.get('query') or data_dict.get('q') or []
    if isinstance(terms, basestring):
        terms = [terms]
    terms = [t.strip() for t in terms if t.strip()]

    if 'fields' in data_dict:
        log.warning('"fields" parameter is deprecated.  '
                    'Use the "query" parameter instead')

    fields = data_dict.get('fields', {})
    offset = data_dict.get('offset')
    limit = data_dict.get('limit')

    q = model.Session.query(model.Tag.name)

    for field, value in fields.items():
        if field in ('tag', 'tags'):
            terms.append(value)

    if not len(terms):
        return [], 0

    for term in terms:
        escaped_term = misc.escape_sql_like_special_characters(
            term, escape='\\')
        q = q.filter(model.Tag.name.ilike('%' + escaped_term + '%'))

    count = q.count()
    q = q.offset(offset)
    q = q.limit(limit)
    return q.distinct(), count
