from sqlalchemy import orm, types, Column, Table, ForeignKey
from ckan.model.meta import metadata, mapper, Session
from ckan.model.package import Package
from ckan.model import domain_object
from ckan import model

import logging
log = logging.getLogger(__name__)

class PackageLikes(domain_object.DomainObject):
    @classmethod
    def get_package_likes(cls, package_id):
        try:
            obj = Session.query(cls).filter_by(
            package_id=package_id).one()
        except orm.exc.NoResultFound:
            return 0
        return obj.num_likes

    @classmethod
    def increase_package_likes(cls, package_id):
        try:
            obj = Session.query(cls).filter_by(
                package_id=package_id).one()
            obj.num_likes = obj.num_likes + 1
        except orm.exc.NoResultFound:
            obj = PackageLikes(package_id=package_id, num_likes=1)
        Session.add(obj)
        Session.commit()

package_likes_table = None
def define_package_likes_table():
    global package_likes_table

    package_likes_table = Table('package_likes', metadata,
                                Column('package_id', types.UnicodeText, ForeignKey('package.id', ondelete='CASCADE'),
                                       primary_key=True, nullable=False),
                                Column('num_likes', types.Integer, default=0),
                                )

    mapper(PackageLikes, package_likes_table,
                properties={
                    'package': orm.relation(Package,
                                            backref=orm.backref('likes', passive_deletes=True, cascade='all, delete, delete-orphan'),
                                            single_parent=True)
                })

def setup():
    # setup package_likes_table
    if package_likes_table is None:
        define_package_likes_table()

    if model.package_table.exists():
        if not package_likes_table.exists():
            package_likes_table.create()
            log.debug('PackageLikes table create')
    else:
        log.debug('PackageLikes table creation deferred')

def migrate_v2_10():
    log.info('Migrating package_likes table to v2.10')
    conn = Session.connection()

    statements = '''
    ALTER TABLE public.package_likes DROP CONSTRAINT package_likes_package_id_fkey;
    ALTER TABLE public.package_likes
        ADD CONSTRAINT package_likes_package_id_fkey FOREIGN KEY (package_id)
        REFERENCES public."package" (id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE CASCADE;
    '''
    conn.execute(statements)

    Session.commit()
    log.info('package_likes table migrated to v2.10')
















