// Monkey-patch broken functionality in Recline.
// Integration point between DGU and Recline. Based on code from CKAN, stripped to requirements.
CKAN.Dgu.resourcePreviewer = (function($,my) {
  my.$dialog = function() { return $('#ckanext-datapreview'); }

  // **Public: Creates a link to the embeddable page.
  //
  // For a given DataExplorer state, this function constructs and returns the
  // url to the embeddable view of the current dataexplorer state.
  my.makeEmbedLink = function(explorerState) {
    var state = explorerState.toJSON();
    state.state_version = 1;

    var queryString = '?';
    var items = [];
    $.each(state, function(key, value) {
      if (typeof(value) === 'object') {
        value = JSON.stringify(value);
      }
      items.push(key + '=' + escape(value));
    });
    queryString += items.join('&');
    return embedPath + queryString;
  };

  // **Public: Loads a data preview**
  //
  // Fetches the preview data object from the link provided and loads the
  // parsed data from the webstore displaying it in the most appropriate
  // manner.
  //
  // link - Preview button.
  //
  // Returns nothing.
  my.loadPreviewDialog = function(resourceData) {
    my.$dialog().html('<h4>Nalaganje podatkov ... <img src="/images/icons/ajaxload-circle.gif" class="loading-spinner" /></h4>');

    function initializeDataExplorer(dataset) {
	var views = [
        {
          id: 'grid',
          label: 'Tabela',
          view: new recline.View.SlickGrid({
            model: dataset,
            state: {
              fitColumns: false,
              gridOptions: {
                //enableColumnReorder: true,
                enableTextSelectionOnCells: true,
                defaultColumnWidth: 120
              }
            }
          })
        },
        {
          id: 'graph',
          label: 'Graf',
          view: new recline.View.Graph({
            model: dataset
          })
        },
        {
          id: 'map',
          label: 'Zemljevid',
          view: new recline.View.Map({
            model: dataset
          })
        }
      ];
      var sidebarViews = [
        {
          id: 'filterEditor', // used for routing
          label: 'Filtri', // used for view switcher
          view: new recline.View.FilterEditor({
            model: dataset
          })
        },
        {
          id: 'fieldsView',
          label: 'Polja',
          view: new recline.View.Fields({
            model: dataset
          })
        }
      ];

      var dataExplorer = new recline.View.MultiView({
        el: my.$dialog(),
        model: dataset,
        views: views,
//        config: {
//          readOnly: true
//        },
	sidebarViews: sidebarViews,
//	sidebarViews: [],
//        state: {
//          query: dataset.queryState
//        }	
      });
      // Upon failure of the datapreivew to load...
      // 1. Hide the controls
      // 2. Hide the space reserved for the preview itself.
      // 3. Disable the Embed button
      dataExplorer.model.bind('query:fail', function(error) {
        $('#ckanext-datapreview .data-view-container').hide();
        $('#ckanext-datapreview .header').hide();
        $('.preview-header .btn').hide();
      });

      // -----------------------------
      // Setup the Embed modal dialog.
      // -----------------------------

      // embedLink holds the url to the embeddable view of the current DataExplorer state.
      var embedLink = $('.embedLink');

      // embedIframeText contains the '<iframe>' construction, which sources
      // the above link.
      var embedIframeText = $('.embedIframeText');

      // iframeWidth and iframeHeight control the width and height parameters
      // used to construct the iframe, and are also used in the link.
      var iframeWidth = $('.iframe-width');
      var iframeHeight = $('.iframe-height');

      // Update the embedLink and embedIframeText to contain the updated link
      // and update width and height parameters.
      function updateLink() {
        var link = my.makeEmbedLink(dataExplorer.state);
        var width = iframeWidth.val();
        var height = iframeHeight.val();
        link += '&width='+width+'&height='+height;

        // Escape '"' characters in {{link}} in order not to prematurely close
        // the src attribute value.
        embedIframeText.val(Mustache.render('<iframe frameBorder="0" width="{{width}}" height="{{height}}" src="{{link}}"></iframe>',
                                       {
                                         link: link.replace(/"/g, '&quot;'),
                                         width: width,
                                         height: height
                                       }));
        embedLink.attr('href', link);
      }

      // Bind changes to the DataExplorer, or the two width and height inputs
      // to re-calculate the url.
      dataExplorer.state.bind('change', updateLink);
      for (var i=0; i<dataExplorer.pageViews.length; i++) {
        dataExplorer.pageViews[i].view.state.bind('change', updateLink);
      }

      iframeWidth.change(updateLink);
      iframeHeight.change(updateLink);

      // Initial population of embedLink and embedIframeText
      updateLink();

      // Finally, since we have a DataExplorer, we can show the embed button.
      $('.preview-header .btn').show();

      // will have to refactor if this can get called multiple times
      Backbone.history.start();
    }

    // 4 situations
    // a) have a webstore_url
    // b) csv or xls (but not webstore)
    // c) can be treated as plain text
    // d) none of the above but worth iframing (assumption is
    // that if we got here (i.e. preview shown) worth doing
    // something ...)
    var tabular_formats =  {'csv':'', 'xls':'', 'xlsx':'', 'tsv':'', 'ods':''};
    
    resourceData.formatNormalized = my.normalizeFormat(resourceData.format);
    var tabular = resourceData.formatNormalized in tabular_formats;
    if (!tabular && resourceData.qa !== undefined &&
        resourceData.qa.format !== undefined && resourceData.qa.format !== null) {
      var qaFormat = my.normalizeFormat(resourceData.qa.format);
      if (qaFormat in tabular_formats) { // if there is any chance that it is tabular, we mark it so
        tabular = true;
        resourceData.formatNormalized = qaFormat;
      }
    }
    resourceData.url  = my.normalizeUrl(resourceData.url);
    
    var tmp = resourceData.url.split('/');
    tmp = tmp[tmp.length - 1];
    tmp = tmp.split('?'); // query strings
    tmp = tmp[0];
    var ext = tmp.split('.');
    var formatByFilename = '';
    if (ext.length > 1) {
      formatByFilename = ext[ext.length-1];
    }
    
    if (resourceData.formatNormalized == '' && formatByFilename != '') {
      resourceData.formatNormalized = formatByFilename;
    }

    if (formatByFilename in {'zip':'','rar':'','gz':'','tar':''} && !resourceData.datastore_active) {
      var message = tabular ? 'Za vpogled prenesite datoteko ali zaprosite objavitelja za vklop polnega predogleda.' : '';
      my.showError({
        title: 'Predogled stisnjene datoteke ni mogoč.',
        message: message
      });
    }
    else if (resourceData.webstore_url) {
      resourceData.elasticsearch_url = '/api/data/' + resourceData.id;
      var dataset = new recline.Model.Dataset(resourceData, 'elasticsearch');
      initializeDataExplorer(dataset);
    }
    else if (resourceData.formatNormalized in {'csv': '', 'xls': '', 'xlsx': '', 'tsv': '', 'ods': ''}) {
      // set format as this is used by Recline in setting format for DataProxy
      resourceData.format = resourceData.formatNormalized;
      var dataset = new recline.Model.Dataset({
        resource_id: resourceData.id,
        format: resourceData.format,
        url: resourceData.url,
//          backend: 'ckan',
        backend: !resourceData.datastore_active ? 'dataproxy': 'ckan',
//        backend: resourceData.datastore_active == undefined || resourceData.datastore_active ? 'ckan' : 'dataproxy',
//	  size: resourceData.archiver.size || 1000
//        size: resourceData.archiver !== undefined? resourceData.archiver.size: 1000
      });
      dataset.fetch().fail(function(error) {
        my.showError(error);
      });
      initializeDataExplorer(dataset);
      dataset.trigger('query:start');
    }
    /*
    else if (resourceData.formatNormalized in {
        'n3': '',
        'n-triples': '',
        'turtle': '',
        'plain': '',
        'txt': ''
        }) {
      var _url = '/data/preview/' + resourceData.id + '?type=plain';
      my.getResourceDataDirect(_url, function(data) {
        my.showPlainTextData(data);
      });
    }
    */
    else if (resourceData.formatNormalized in {'html':'', 'htm':''}) {
      // we displays a fullscreen dialog with the url in an iframe.
      my.$dialog().empty();
      var el = $('<iframe></iframe>');
      el.attr('src', resourceData.url);
      el.attr('width', '100%');
      el.attr('height', '100%');
      // Change this to be a specific element for #977
      $('#ckanext-html-preview').append(el);
      //my.$dialog().append(el);
    }
    // images
    else if (resourceData.formatNormalized in {'png':'', 'jpg':'', 'jpeg':'', 'gif':''}
        ||  resourceData.resource_type=='image') {
      // we displays a fullscreen dialog with the url in an iframe.
      my.$dialog().empty();
      var el = $('<img />');
      el.attr('src', resourceData.url);
      el.css('max-width', '100%');
      el.css('border', 'solid 4px black');
      $('#ckanext-image-preview').append(el);
      //my.$dialog().append(el);
    }
    else if (resourceData.formatNormalized in {'xml':''}) {
	  // we displays a fullscreen dialog with the url in an iframe.
	  my.$dialog().empty();
	  
	  var el = $('<iframe></iframe>');
      el.attr('src', resourceData.url);
      el.attr('type','application/xml');
      el.attr('width', '100%');
      el.attr('height', '100%');
	  
	  //var el = $('<img />');
	  //el.attr('src', resourceData.url);
	  //$('#ckanext-xml-preview').innerHTML = $('#ckanext-xml-preview').innerHTML + 'Extra stuff';
	  
	  
	  $('#ckanext-xml-preview').append(el);
	  //my.$dialog().append(el);
	}
	else if (resourceData.formatNormalized in {'json':''} ||  resourceData.resource_type=='json') {
      // we displays a fullscreen dialog with the url in an iframe.
      my.$dialog().empty();

      var el = $('<iframe></iframe>');
        el.attr('src', resourceData.url);
        el.attr('type','application/json');
        el.attr('width', '100%');
        el.attr('height', '100%');

      $('#ckanext-json-preview').append(el);
    }
    else if (resourceData.formatNormalized in {'txt':''} ||  resourceData.resource_type=='plain' ||  resourceData.resource_type=='txt') {
  	  // we displays a fullscreen dialog with the url in an iframe.
  	  my.$dialog().empty();
  	  
  	  var el = $('<iframe></iframe>');
        el.attr('src', resourceData.url);
        el.attr('type','text/plain');
        el.attr('width', '100%');
        el.attr('height', '100%');
  	  
  	  //var el = $('<img />');
  	  //el.attr('src', resourceData.url);
  	  //$('#ckanext-xml-preview').innerHTML = $('#ckanext-xml-preview').innerHTML + 'Extra stuff';
  	  
  	  
  	  $('#ckanext-txt-preview').append(el);
  	  //my.$dialog().append(el);
  	}
    else {
      // Cannot reliably preview this item - with no mimetype/format information,
      // can't guarantee it's not a remote binary file such as an executable.
      my.showError({
        title: 'Predogled za podatkovni tip ' + resourceData.formatNormalized + ' ni na voljo.',
        message: ''
      });
    }
  };

  // Public: Requests the formatted resource data from the webstore and
  // passes the data into the callback provided.
  //
  // preview - A preview object containing resource metadata.
  // callback - A Function to call with the data when loaded.
  //
  // Returns nothing.
  my.getResourceDataDirect = function(url, callback) {
    // $.ajax() does not call the "error" callback for JSONP requests so we
    // set a timeout to provide the callback with an error after x seconds.
    var timeout = 30000;
    var timer = setTimeout(function error() {
      callback({
        error: {
          title: 'Napaka',
          message: 'Podatkovni strežnik ni podal odgovora po ' + (timeout / 1000) + ' sekundah'
        }
      });
    }, timeout);

    // We need to provide the `cache: true` parameter to prevent jQuery appending
    // a cache busting `={timestamp}` parameter to the query as the webstore
    // currently cannot handle custom parameters.
    $.ajax({
      url: url,
      cache: true,
      dataType: 'json',
      success: function(data) {
        clearTimeout(timer);
        callback(data);
      },
      error: function(err) {
          clearTimeout(timer);
          callback({
            error: {
              title: 'Napaka',
              message: 'Napaka pri obdelavi zahtevka'
            }
          });
      }
    });
  };

  // Public: Displays a String of data in a fullscreen dialog.
  //
  // data    - An object of parsed CSV data returned by the webstore.
  //
  // Returns nothing.
  my.showPlainTextData = function(data) {
    if(data.error) {
      my.showError(data.error);
    } else {
      var content = $('<pre></pre>');
      for (var i=0; i<data.data.length; i++) {
        var row = data.data[i].join(',') + '\n';
        content.append(my.escapeHTML(row));
      }
      my.$dialog().html(content);
      if (data.extra_text) {
          $("#ckanext-datapreview-extra").html(data.extra_text);
      }
      if (data.archived) {
          $("#ckanext-datapreview-source").html(data.archived);
      }
    }
  };

  my.showError = function (error) {
    var _html = _.template(
        '<div class="panel error"><div class="panel-heading"><strong><%= title %></strong></div><div class="panel-body"><%= message %></div></div>')(error);
    my.$dialog().html(_html);
  };

  my.normalizeFormat = function(format) {
    var out = format.toLowerCase();
    out = out.split('/');
    out = out[out.length-1];
    return out;
  };

  my.normalizeUrl = function(url) {
	var toHttps = $("div.resource-preview").hasClass("using-https");
    if (url.indexOf('https') === 0) {
      if (!toHttps)
        return 'http' + url.slice(5);
    } else if (url.indexOf('http') === 0) {
      if (toHttps)
        return 'https' + url.slice(4);
    }
    return url;
  }

  // Public: Escapes HTML entities to prevent broken layout and XSS attacks
  // when inserting user generated or external content.
  //
  // string - A String of HTML.
  //
  // Returns a String with HTML special characters converted to entities.
  my.escapeHTML = function (string) {
    return string.replace(/&(?!\w+;|#\d+;|#x[\da-f]+;)/gi, '&amp;')
                 .replace(/</g, '&lt;').replace(/>/g, '&gt;')
                 .replace(/"/g, '&quot;')
                 .replace(/'/g, '&#x27')
                 .replace(/\//g,'&#x2F;');
  };

  return my;
})(jQuery, CKAN.Dgu.resourcePreviewer || {});

CKAN.Dgu.initDatastoreButton = function(resource_id) {
  $('#upload_to_datastore').click(function () {
    $('#datastore-spinner').show();
    $.ajax({
      type: 'POST',
      url: '/api/action/quick_datapusher_submit',
      data: '{"resource_id":"' + id +'"}'
    })
    .done(function() {
      $('#upload_to_datastore').replaceWith('<div class="response-info"><strong>Obdelava je sprožena. Čez nekaj časa lahko osvežite stran in preverite, če so prikazani vsi podatki.</strong></div>');
    }).fail(function(error) {
      if (error.responseText !== undefined) {
        error = error.responseText;
      }
      $('#upload_to_datastore').replaceWith('<div class="response-info"><strong>Proženje obdelave je spodletelo z napako '+error+', prosimo da obvestite skrbnika.</strong></div>');
    }).always(function() {
      $('#datastore-spinner').hide();
    });
  });
}

